##########################################################################
#
#  File:      dossim.mak
#
#  Abstract : Makefile for Building a RenderWare application with
#             the Watcom C compiler version 9.5 or 10.0
#
##########################################################################
#
# This file is a product of Criterion Software Ltd.
#
# This file is provided as is with no warranties of any kind and is
# provided without any obligation on Criterion Software Ltd. or
# Canon Inc. to assist in its use or modification.
#
# Criterion Software Ltd. will not, under any
# circumstances, be liable for any lost revenue or other damages arising
# from the use of this file.
#
# Copyright (c) 1995 Criterion Software Ltd.
# All Rights Reserved.
#
# RenderWare is a trademark of Canon Inc.
#
###########################################################################
#
# To build the dossim.exe executable type
#
# wmake /f dossim.mak


# Build flags

RWDEBUG  = 0                # 1 = Link with debugging RenderWare library
                            # 0 = Link with production RenderWare library

RWFIXED  = 1                # 1 = Link with fixed point RenderWare library
                            # 0 = Link with floating point RenderWare library

# Define the location of the RenderWare library and include files

RWINCDIR = ..\..\..\..\include
RWLIBDIR = ..\..\..\..\lib

# Set fixed or float flags

!ifeq RWFIXED 1
FX_CFLAGS = -DRWFIXED -fpc
!else
FX_CFLAGS = -DRWFLOAT -7
!endif

# Determine which library to link against based on the options above

!ifeq RWFIXED 1
!ifeq RWDEBUG 1
RWLIB = $(RWLIBDIR)\rwdrxd.lib
!else # RWDEBUG
RWLIB = $(RWLIBDIR)\rwdrxp.lib
!endif # RWDEBUG
!else # RWFIXED
!ifeq RWDEBUG 1
RWLIB = $(RWLIBDIR)\rwdrld.lib
!else # RWDEBUG
RWLIB = $(RWLIBDIR)\rwdrlp.lib
!endif # RWDEBUG
!endif # RWFIXED

# C Compiler options

CC        = *wcc386
CFLAGS    = $(FX_CFLAGS) $(DB_CFLAGS) -I$(RWINCDIR) -oneatx -w4 -j -zq -ei -s
COUT      = -fo

# Linker options

LINK      = wlink
CLFLAGS   = op q op st=32768 $(CDB_LFLAGS)
BIND	  = wbind

# Rules

.c.obj:
	$(CC) $(CFLAGS) $(COUT)$*.obj $?

# Define the name for the executable

BASENAME = dossim

# Define the object files that make up this program

OBJLIB = dossim.obj $(RWLIB)

# Build the executable. We use an intermediate link file to simplify this
# procedure and to allow arbitrary length link lines

all:	$(BASENAME).exe .SYMBOLIC

$(BASENAME).exe: $(BASENAME).lnk $(OBJLIB)
	$(LINK) @$*
        del $(BASENAME).lnk
        del *.err
        del *.obj

$(BASENAME).lnk:
          @%create $^@
          @%append $^@ $(CLFLAGS)
          @%append $^@ NAME $^&
          @for %i in ($(OBJLIB)) do @%append $^@ FILE %i

# Clean up any intermediate build files leaving the executable
clean:  .SYMBOLIC
        del $(BASENAME).lnk
	del *.err
	del *.obj
